/******************************************************************/
/* filename: user.c */
/* version : 1.00  */
/* author  : Jan Feenstra */
/******************************************************************/

#include "cp.h"
#include "cfpe.h"
 /* MAXPARTNERS */
#include "processConfig.h"
#include "debug.h"

/******************************************************************/
/* External declaration of CSAP pipes [defined in cp.c] */
extern int CSAP_datareq_pipe[2]; /* We use only the [1] pipe for writing. */
extern int CSAP_dataind_pipe[2]; /* We use only the [0] pipe for reading. */

/******************************************************************/
/* External declaration of functions */
extern int decodeData();
extern int encodeData();
extern int decodeTitle();
extern int encodeTitle();
extern int writeTitle();
extern int writeConfid();
extern int encodeConfid();
extern int main();

/******************************************************************/
/* function prototying (forward declaration) */
int user(void);
int entercommandline(char *cmdln, char prompt[USERTITLELENGTH]);
int processcommandline(char **cmdln);
int quit(char *cmdln);
int join(char *cmdln, char prompt[USERTITLELENGTH]);
int help(char *cmdln);
int leave(char *cmdln);
int receive(char *cmdln);
int _send(char *cmdln);

/******************************************************************/
/* Declaration of local variables */
char prompt[USERTITLELENGTH];

/******************************************************************/

int user()
{
 char commandline[300]; /* allocate 'too manny space!!! */
 char *p;
 int command;
 int idx;
 int rvalue=0;
  
 Msg(User,"Conference Protocol\n");
 strcpy(prompt, "");
 do
 {
  Msg(User,"%s> ",prompt); 
  entercommandline((char *)&commandline,prompt);
  p = commandline;
  command = processcommandline(&p);

  switch (command)
  {
   case 0: {quit(commandline); break;}
   case 1: {join(p,prompt); break;}
   case 2: {_send(p); break;}
   case 3: {rvalue = receive(commandline); break; }
   case 4: {leave(prompt); break;}
   case 5: {help(commandline); break;}
   default: {_send(p);; break;}
  }
 } while (command && (rvalue>=0));
 return rvalue;
}

/******************************************************************/

int entercommandline(char *cmdln, char prompt[USERTITLELENGTH])
{
 fd_set read_mask;

 FD_ZERO(&read_mask);
 FD_SET(CSAP_dataind_pipe[0], &read_mask);
 FD_SET(1, &read_mask);

 select(32, &read_mask, (fd_set *) 0, (fd_set *) 0, (struct timeval *)0);

 if (FD_ISSET(1, &read_mask))
 {
  gets(cmdln); 
 }
 else
 if (FD_ISSET(CSAP_dataind_pipe[0], &read_mask))
 {
  strcpy(cmdln, "r");
 }
 return(0);
}
/******************************************************************/

int processcommandline(char **cmdln)
{
 int cmd;
 switch (**cmdln) {
  case 'j' : 
  case 'J' : cmd=1; break;
  case 's' :
  case 'S' : cmd=2; break;
  case 'r' : 
  case 'R' : cmd=3; break;
  case 'l' : 
  case 'L' : cmd=4; break;
  case 'q' : 
  case 'Q' : cmd=0; break;
  case 'h' :
  case 'H' : cmd=5; break;
  default  : cmd=6; break;
 }
 while ((**cmdln) && (**cmdln!=' ') && (**cmdln!='\t')) (*cmdln)++;
 while ((**cmdln) && ((**cmdln==' ') || (**cmdln=='\t'))) (*cmdln)++;

 return cmd;
}
/******************************************************************/

int quit(char *cmdln)
{ 
 return(0);
}
/******************************************************************/

int join(char *cmdln, char prompt[USERTITLELENGTH])
{
 struct join_cdu_Type cdu; 
 char usertitle[USERTITLELENGTH+1];
 char confid[CONFIDLENGTH+1];
 char rawstr1[(USERTITLELENGTH+1)*4];
 char rawstr2[(CONFIDLENGTH+1)*4];
 int len=0;

 cdu.type = JOIN;
 
 if (sscanf(cmdln, "%s", &rawstr1)<=0)
  rawstr1[0]='\0';
 raw2cooked(&usertitle, rawstr1);
 strcpy(prompt, usertitle);
 encodeTitle(usertitle, &cdu.usertitle);

 cmdln += strlen(rawstr1);

 if (sscanf(cmdln, "%s\n", &rawstr2)<=0)
  rawstr2[0]='\0';
 raw2cooked(&confid, rawstr2);
 encodeConfid(confid, &cdu.confid);

 write(CSAP_datareq_pipe[1], (char *)&cdu, sizeof(struct join_cdu_Type));
 return 0;
}
/******************************************************************/

int raw2cooked(char usertitle[USERTITLELENGTH+1], char rawstr[(USERTITLELENGTH+1)*4])
/* The codes may be larger than 4 characters in total, for every code! */
{
 int len=USERTITLELENGTH;
 int i=0;
 int j=0;
 
 usertitle[USERTITLELENGTH] = '\0';
 while (len)
 {
  switch (rawstr[j])
  {
   case '\0':
   case '\t':
   case ' ':
   {
    usertitle[i] = '\0';
    return 0;
   }
   case '\\':
   {
    j++; if (j==((USERTITLELENGTH+1)*4))
	 {
      Debug(User, "Test 002");
	  break;
     }
    switch (rawstr[j])
    {
     case '\\':
     case ' ':
     {
      Debug(User, "Test 003");
      usertitle[i]=rawstr[j];
      len--;
      i++;
      j++; 
      if (j==((USERTITLELENGTH+1)*4))
      {
       Debug(User, "Test 004");
       usertitle[i] = '\0';
       return 0;
      }
      break;
     }
     case '0':
     case '1':
     case '2':
     case '3':
     case '4':
     case '5':
     case '6':
     case '7':
     case '8':
     case '9':
     {
      int number;
      sscanf((char *)&rawstr[j],"%u",&number);
	  usertitle[i] = (char)number;
      i++;
      len--;
      while ((j<((USERTITLELENGTH+1)*4))&&(('0'<=rawstr[j]) \
             && (rawstr[j]<='9'))) {j++;}
      if (j==((USERTITLELENGTH+1)*4))
       return 0;
      else
       break;
     }
     default:
     {
      break;
     }
    }
    break;
   }
   default :
   {
    usertitle[i]=rawstr[j];
    len--;
    i++;
    j++; if (j==((USERTITLELENGTH+1)*4)) break;
   }
  }
 }
}
 
/******************************************************************/

int _send(char *cmdln)
{
 struct datareq_cdu_Type cdu; 
 char data[DATALENGTH+1];

 cdu.type = DATAREQ;
 encodeData(cmdln, &cdu.length, &cdu.data);

 write(CSAP_datareq_pipe[1], (char *)&cdu, sizeof(struct datareq_cdu_Type));
 return 0;
}
/******************************************************************/

int receive(char *cmdln)
{
 struct dataind_cdu_Type cdu; 
 char usertitle[USERTITLELENGTH+1];
 char data[DATALENGTH+1];
 int read_result;
 read_result = read(CSAP_dataind_pipe[0], (char *)&cdu, \
    sizeof(struct dataind_cdu_Type));
 if (read_result < 0 )
 {
  Error(User, "user.c: Reading from CSAP_dataind_pipe[0]!");
  return(-1);
 }
 else if (read_result == 0) 
 {
  return(-2); /* EOF */
 }
 else 
 {
  switch (cdu.type)
  {
   case DATAIND:
   {
    decodeTitle(cdu.usertitle, &usertitle);
    decodeData(cdu.length, cdu.data, &data);

    Msg(User,"\n%s> ", usertitle);
    puts(data);
    break;
   }
   case JOIN:
   case DATAREQ:
   case LEAVE: 
   {
    Warning(User,"user.c: Reading from CSAP_dataind_pipe[0]: Undesired cdu.");
    break;
   }
   default:
   {
    Error(User,"user.c: Reading from CSAP_dataind_pipe[0]: Unknown cdu.");
    return(-1);
   }
  }
 }
 return 0;
}
/******************************************************************/

int leave(char prompt[USERTITLELENGTH])
{
 struct leave_cdu_Type cdu; 
 cdu.type = LEAVE;
 write(CSAP_datareq_pipe[1], (char *)&cdu, sizeof(struct leave_cdu_Type));
 strcpy(prompt,"");
 return 0;
}
/******************************************************************/

int help(char *cmdln)
{
 Msg(User,"\nCommands:\n");
 Msg(User,"Join\t- join a conference. [join usertitle confid]\n");
 Msg(User,"Send\t- send message to a conference. [send message]\n");
 Msg(User,"Leave\t- leave conference. [leave ]\n");
 Msg(User,"Help\t- help about the available commands.\n");
 Msg(User,"Quit\t- quit program.\n");
 return 0;
}
/******************************************************************/

